#include <cstdio>
#include <cstring>
#include <string>

/* Finnish name conjugation design tester (1-5 character version)
 * for Chrono Trigger Finnish translation
 *
 * Homepage of the project: http://bisqwit.iki.fi/ctfin/
 *
 * Conjugates names.
 * Doesn't know of any exceptions. (Mki?)
 * Doesn't handle plurals. (Names don't usually come in plurals.)
 *
 * Algorithm:
 *     Vowel means character is in aeiouy.
 *     For the word to be conjugated, check these variables:
 *       Front
 *         - Initially true, unless the name starts with h/k/q/z
 *         - True if last aouy0-9 is in y14579
 *         - False if last aouy0-9 is in aou02368
 *       HasVowel
 *         - True if last aouy is in aouy
 *       LastChar
 *         - Last char of word
 *       LastChar2
 *         - Second last char of word (or first)
 *       LastChar3
 *         - Third last char of word
 *       Size
 *         - Length of the name
 *     If conjugating to PARTitive:
 *       Display the full name.
 *       If size=1 or !HasVowel
 *         Display ':'
 *         If LastChar is in flmnlrsx, display '' and set Front=true
 *         Else If LastChar is z, display 'a' and set Front=false
 *         Else display 't'
 *       Else If LastChar is 's' and lastchar2 is not vowel,
 *         Display 't'
 *       Else if LastChar is not vowel,
 *         Display 'i'
 *     Else
 *       I
 f size=1 or !HasVowel
 *         Display the full name and ':'
 *       Else If LastChar is 's' and lastchar2 is not vowel,
 *         Display the full name and 'kse'
 *       Else if LastChar is vowel
 *         Display the full name and 'i'
 *       Else If size <= 2
 *         Display the full name
 *       Else If LastChar is vowel and lastchar2 = lastchar3
 *                                 and lastchar2 in kpt,
 *         Display the name except second last character
 *       Else
 *         Display the full name
 *
 *     If conjugating to ACCusative
 *       Display 'n'
 *     Else If conjugating to PARTitive
 *       Display '' or 'a' depending on Front
 *     Else if conjugating to GENitive
 *       Display 'n'
 *     Else if conjugating to ADEssiive
 *       Display 'll'
 *       Display '' or 'a' depending on Front
 *     Else if conjugating to ALLAtive
 *       Display 'lle'
 *
 * Copyright (C) 1992,2003 Bisqwit (http://iki.fi/bisqwit/)
 */

// These functions do not handle plural (never needed
// with names) or alphanumeric codenames.

static char NameBuf[6];

enum specialchars
{
    NAME=1,
    ACC,         //   akkusatiivi = the object of action that affects the target as whole
    PART,        //   partitiivi  = the object of action that affects the target
    GEN,         //   genetiivi   = owner of something, in English usually "'s"
    ADE,         //   adessiivi   = owner/container of something, in English "has" or "in" or "on"
    ALLA         //   allatiivi   = target of something, in English sometimes as "to" or "for" preposition
};

template<typename T>
static const std::string operator+ (const T s, specialchars k)
{
    std::string result(s);
    result += (char)k;
    return result;
}

bool isvowel(const char c)
{
    switch(c)
    {
        case 'a': case 'e': case 'i': case 'o':
        case 'u': case 'y': case '': case '':
        case 'A': case 'E': case 'I': case 'O':
        case 'U': case 'Y': case '': case '':
        case '': case '': case '': case '':
            return true;
    }
    return false;
}

/////////////////////////////////////

static void RealDispChar(char c)
{
    std::putchar(c);
}

// This function eats one character from the
// script and outputs various amount of characters.

static void DispChar(char c)
{
    static char ConjuCode = 0;
    char ConjuWord[6];
    
    // NAME: Test for conjugate
    if(c == NAME) { std::strcpy(ConjuWord, NameBuf); goto ConjugateTest; }

    if(c == ACC
    || c == GEN
    || c == PART
    || c == ADE
    || c == ALLA
      )
        ConjuCode = c;
    else
    {
        ConjuCode = 0;
        RealDispChar(c);
    }
    return;

ConjugateTest:
    unsigned size=0;
    while(ConjuWord[size]) ++size;
    
    if(!ConjuCode)
    {
        for(unsigned a=0; a<size; ++a)RealDispChar(ConjuWord[a]);
        return;
    }
    
    char lastchar  = ConjuWord[size > 1 ? size-1 : 0];
    char lastchar2 = ConjuWord[size > 2 ? size-2 : 0];
    char lastchar3 = ConjuWord[size > 3 ? size-3 : 0];
    
    bool hasvowel = false;
    bool yla = ConjuWord[0] != 'h' && ConjuWord[0] != 'H'
            && ConjuWord[0] != 'k' && ConjuWord[0] != 'K'
            && ConjuWord[0] != 'q' && ConjuWord[0] != 'Q'
            && ConjuWord[0] != 'z' && ConjuWord[0] != 'Z';
    
    // Description: If the last vowel of the body is a/o/u,
    // we will use the lower (ala) suffix. If the last vowel
    // of the body is //y, we'll use the upper (yl) suffix.
    // If the last vowel is any other vowel, check the previous
    // vowel. Otherwise we'll use the default suffix (yl).
    for(unsigned a=size; a-->0; )
    {
        char ch = ConjuWord[a];
        if(ch == 'a' || ch == 'o' || ch == 'u'
        || ch == 'A' || ch == 'O' || ch == 'U') { yla=false; hasvowel=true; break; }
        if(ch == '' || ch == '' || ch == 'Y'
        || ch == '' || ch == '' || ch == 'y') { yla=true; hasvowel=true; break; }
        if(ch == '1' || ch == '4' || ch == '5'
        || ch == '7' || ch == '9') { yla=true; break; }
        if(ch == '0' || ch == '2' || ch == '3'
        || ch == '6' || ch == '8') { yla=false; break; }
    }
    

    if(ConjuCode == PART)
    {
    	for(unsigned a=0; a<size; ++a)RealDispChar(ConjuWord[a]);
        // Case "hard body"
        if(size == 1 || !hasvowel)
        {
            // Abbreviation. This isn't very good but there's no better.
            // Examples: X,a -> X:
            //           A,a -> A:ta
            //           P,a -> P:t
            RealDispChar(':');
            if(lastchar == 'f' || lastchar == 'F'
            || lastchar == 'l' || lastchar == 'L'
            || lastchar == 'm' || lastchar == 'M'
            || lastchar == 'n' || lastchar == 'N'
            || lastchar == 'l' || lastchar == 'L'
            || lastchar == 'r' || lastchar == 'R'
            || lastchar == 's' || lastchar == 'S'
            || lastchar == 'x' || lastchar == 'X')
            {
                yla = true;
                RealDispChar('');
            }
            else if(lastchar == 'z' || lastchar == 'Z')
            {
                yla = false;
                RealDispChar('a');
            }
            else
                RealDispChar('t');
        }
        else if((lastchar == 'S' || lastchar == 's')
              && isvowel(lastchar2))
        {
            // If the word ends with s, add "t"
            // Example: Magus,a -> Magusta
            RealDispChar('t');
        }
        else if(!isvowel(lastchar))
        {
            // If the word does not end with vowel, add "i"
            // Example: John,a -> Johnia
            RealDispChar('i');
        }
    }
    else
    {
        // Case "soft body"
        if(size == 1 || !hasvowel)
        {
            // Abbreviation. This isn't very good but there's no better.
            // Example: X,lle -> X:lle
            for(unsigned a=0; a<size; ++a)RealDispChar(ConjuWord[a]);
            RealDispChar(':');
        }
        else if((lastchar == 'S' || lastchar == 's')
              && isvowel(lastchar2))
        {
            // If the word ends with vowel+s, replace s with "kse"
            // Example: Magus,lle -> Magukselle
            for(unsigned a=0; a<size-1; ++a)RealDispChar(ConjuWord[a]);
            RealDispChar('k');
            RealDispChar('s');
            RealDispChar('e');
        }
        else if(!isvowel(lastchar))
        {
            // If the word ends with non-vowel, add "i"
            // Example: John,lle -> Johnille
            for(unsigned a=0; a<size; ++a)RealDispChar(ConjuWord[a]);
            RealDispChar('i');
        }
        else if(size <= 2)
        {
            for(unsigned a=0; a<size; ++a)RealDispChar(ConjuWord[a]);
        }
        else if(isvowel(lastchar)
            && lastchar2 == lastchar3
            && (lastchar2 == 'k' || lastchar2 == 'p' || lastchar2 == 't'
             || lastchar2 == 'K' || lastchar2 == 'P' || lastchar2 == 'T')
               )
        {
            // If the word ends with hardconsonantdouble + vowel,
            // Remove one consonant.
            // Example: Mikko,lle -> Mikolle
    
            for(unsigned a=0; a<size-2; ++a)RealDispChar(ConjuWord[a]);
            RealDispChar(lastchar);
        }
        else
        {
            // Otherwise it's just that.
            // Example: Mika,lle -> Mikalle
            for(unsigned a=0; a<size; ++a)RealDispChar(ConjuWord[a]);
        }
    }

    if(ConjuCode == ACC)
    {
        RealDispChar('n');
    }
    else if(ConjuCode == PART)
    {
        RealDispChar(yla ? '' : 'a');
    }
    else if(ConjuCode == GEN)
    {
        RealDispChar('n');
    }
    else if(ConjuCode == ADE)
    {
        RealDispChar('l');
        RealDispChar('l');
        RealDispChar(yla ? '' : 'a');
    }
    else if(ConjuCode == ALLA)
    {
        RealDispChar('l');
        RealDispChar('l');
        RealDispChar('e');
    }

    ConjuCode = 0;
}

static void Hoitele(const char *s)
{
    while(*s)
        DispChar(*s++);
}

int main(int argc, const char *const *argv)
{
	if(!argv[1])
	{
		std::fprintf(std::stderr,
		    "Usage: taipus <name>\n"
            "Name is 1-5 characters long name, consisting of characters a-z,,,,.\n");
		return 0;
	}
    std::strncpy(NameBuf, argv[1], 5);
    NameBuf[5] = 0;
    
    /* All different test cases:
     *    mikko, mkki
     *    zgw, r1234
     *    magus, ters
     *    simo, lts
     *
     * Bugi: mki, ksi
     */
    
    Hoitele((
        "Pasi nki " + ACC + NAME + "\n"
        "Pasi kiitt " + PART + NAME + "\n"
        + GEN + NAME + " kello on rikki\n"
        + ADE  + NAME + " on kissa\n"
        + ALLA + NAME + " soitettiin monta kertaa\n").c_str());
}
